using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using Microsoft.Office.Excel.Server.Udf;
using Microsoft.SharePoint;
using System.Security.Principal;
using System.Runtime.InteropServices;				// For Client Compat
using Microsoft.Win32;								// For Client Compat

namespace UDF
{
    [UdfClass]
    [Guid(Class1.ClsId)]							// For Client Compat
    [ProgId(Class1.ProgId)]							// For Client Compat
    [ClassInterface(ClassInterfaceType.AutoDual)]	// For Client Compat
    [ComVisible(true)]								// For Client Compat
    public class Class1
    {
        #region CLIENT COMPATIBILITY VARIABLES

        const string ClsId = "5290B6FB-9FD0-42a1-8FDC-5EBFD97CC399";	// For Client Compat
        const string ProgId = "UDF.Class1";	                            // For Client Compat

        #endregion
        #region CLIENT COMPATIBILITY

        [ComRegisterFunction]	                            // For Client Compat
        public static void RegistrationMethod(Type type)	// For Client Compat
        {
            // Only add stuff to the registration 
            // if it's this class that's being registered.
            if (typeof(Class1) != type)
            {
                return;
            }
            // Add "Programmable" under our key.
            RegistryKey key = Registry.ClassesRoot.CreateSubKey(
                   "CLSID\\{" + ClsId + "}\\Programmable");
            key.Close();
        }

        [ComUnregisterFunction]	                            // For Client Compat
        public static void UnregisterationMethod(Type type)	// For Client Compat
        {
            // Only add stuff to the registration 
            // if it's this class that's being registered.
            if (typeof(Class1) != type)
            {
                return;
            }
            // Add "Programmable" under our key.
            Registry.ClassesRoot.DeleteSubKey("CLSID\\{" + ClsId + "}\\Programmable");
        }

        #endregion
        #region GETSHAREPOINTVIEW

        [UdfMethod(IsVolatile = true, ReturnsPersonalInformation=true)]
        [ComVisible(true)]
        public object[,] getSharepointView(string serverName, string siteName, string listName, string viewName, bool getTitles)
        {
        	try
     	    	{


	  		// We should impersonate the current user viewing EWR... not the UDF manager...
            	        using (WindowsImpersonationContext wiContext = impersonateUser())
        		{
   	    	 		// Grab Sharepoint Objects
    	   			SPSite site = new SPSite(serverName);
  	              		SPList list = site.AllWebs[siteName].Lists[listName];
  	              		SPView view = list.Views[viewName];
              
     	           		// Grab collection of Sharepoint List Values
              	  		SPListItemCollection values = list.GetItems(new SPQuery(view));

                		int titleRow = 0;

                		// Save a row for the titles if requested
                		if (getTitles)
                    			titleRow = 1;

                		int totalRows = values.Count + titleRow;

                		// HACK to get around Excel "By Design" bug 472566 that repeats a row if a 2-d array is really only 1-d.
                		if (totalRows == 1)
                  	  		totalRows++;

                		// Create object array to return to Excel
               			object [,] toExcel = new object[totalRows, view.ViewFields.Count];

                		// Add column titles to array if needed
                		if (getTitles)
                		{
                    			for (int i = 0; i < view.ViewFields.Count; i++)
                    			{
                        			toExcel[0, i] = list.Fields.GetFieldByInternalName(view.ViewFields[i]).Title;
                    			}
                		}

                		int j = titleRow;

                		// Iterate through SPListItems in the view
                		foreach (SPListItem currVal in values)
                		{
                    			// Add every field value of current item to the array that will be returned
                    			for (int i = 0; i < view.ViewFields.Count; i++)
                    			{
                        			// Get the current fueld
                        			SPField field = currVal.Fields.GetFieldByInternalName(view.ViewFields[i]);

                                    // Get the field value as text string.
                                    toExcel[j, i] = field.GetFieldValueAsText(currVal[field.Id]);                  
                    			}
                    			j++;
                		}

                		return toExcel;
				
            		}
		}
            	catch(System.Exception ex)
            	{
                	object[,] error = new object[1, 1];
                	error[0,0] = ex.Message;
                	return error;
            	}

        }
        #endregion
        #region GETSHAREPOINTVIEW HELPERS
        // Causes the current UDF to impersonate the user that loaded it through EWR
	    private WindowsImpersonationContext impersonateUser()
	    {
            WindowsIdentity wi = null;

		    try
		    {
			// BEGIN CLIENT COMPAT. SECTION
        		// Check if code is not running on the server. 
		        // If true, then it is running through a COM interface 
		        // on the client and should not be impersonated.
		        if (System.Diagnostics.Process.GetCurrentProcess()
			            .ProcessName == "EXCEL") 
		        {
		            // COM case (client-only), should not impersonate.
		            return null;
		        }
		        // END CLIENT COMPAT. SECTION


		    	// Get the user identity that loaded Workbook, and impersonate it
		    	wi = (WindowsIdentity)System.Threading.Thread.CurrentPrincipal.Identity;
		    }
		    catch (System.Exception ex)
		    {
                throw new InvalidOperationException("Impersonation failed.");
		    }

		    return wi.Impersonate();
 	    }
        #endregion
    }
}